#include "MatAllocator.h"
#include "globalFuncs.h"

template <typename Float>
QUIC::MatAllocator<Float>::MatAllocator()
{}

template <typename Float>
QUIC::MatAllocator<Float>::~MatAllocator()
{}

template <typename Float>
typename QUIC::MatAllocator<Float>::pitched_pointer QUIC::MatAllocator<Float>::allocate(size_type w, size_type h)
{
	assert(w != 0 && h != 0);
	pitched_pointer p;
	size_type rowWidth = w*sizeof(Float);
	p.pitch = iDivUp(rowWidth, 256)*256;
	try
	{
		p.addr = reinterpret_cast<pointer>(::operator new(h*p.pitch));
	}
	catch (std::bad_alloc&)
	{
		p.addr = NULL;
		setError("Memory allocation failed.");
	}
	return p;
}

template <typename Float>
void QUIC::MatAllocator<Float>::deallocate(pitched_pointer p, size_type width, size_type height)
{
	::operator delete(p.addr);
}

template <typename Float>
QUIC::MatAllocatorGPU<Float>::MatAllocatorGPU()
{}

template <typename Float>
QUIC::MatAllocatorGPU<Float>::~MatAllocatorGPU()
{}

template <typename Float>
typename QUIC::MatAllocator<Float>::pitched_pointer QUIC::MatAllocatorGPU<Float>::allocate(size_type w, size_type h)
{
	pitched_pointer p;
	size_t rowWidth = w*sizeof(Float);
	if (!QUIC_SAFE_CALL(cudaMallocPitch(reinterpret_cast<void **>(&p.addr), &p.pitch, rowWidth, h))) p.addr = NULL;
	return p;
}

template <typename Float>
void QUIC::MatAllocatorGPU<Float>::deallocate(pitched_pointer p, size_type w, size_type h)
{
	QUIC_SAFE_CALL(cudaFree(p.addr));
}

INSTANTIATE_CLASS_TEMPLATE(QUIC::MatAllocator)