#pragma once

#include "Basis.h"
#include "globalFuncs.h"

template <typename Float>
class CosineTree;
template <typename Float>
class LSDistribution;

template <typename Float>
class CTNode : public QUIC::Base
{
public:
	static enum {LEFT = 0, RIGHT = 1};
	CTNode(const QUIC::Uint& nr, CosineTree<Float> * ct, const typename Basis<Float>::iterator& iter);
	CTNode(const QUIC::Uint& nr, CosineTree<Float> * ct);
	virtual ~CTNode();
	const QUIC::Uint& operator[](const QUIC::Uint& i) const;
	const QUIC::Uint& size() const;
	virtual CTNode * split();
	virtual void getBasisVector(Float * c);
	void populateChildRows(QUIC::Uint * labels, const QUIC::Uint& blockSize, const QUIC::Uint& nBlocks);
	void initRows();
	QUIC::Uint nSamples() const;
	const LSDistribution<Float>& LS() const;
	const typename Basis<Float>::iterator& getBasisVector() const;
	void deleteBasisVector();
protected:
	CTNode<Float> * children;
	CosineTree<Float> * cosineTree;
	typename Basis<Float>::iterator * basisVector;
	QUIC::Uint nRows;
	QUIC::Uint * rows;
};

template <typename Float>
class CTNodeGPU : public CTNode<Float>
{
public:
	CTNodeGPU(const QUIC::Uint& nr, CosineTree<Float> * ct, const typename Basis<Float>::iterator& iter);
	CTNodeGPU(const QUIC::Uint& nr, CosineTree<Float> * ct);
	~CTNodeGPU();
	CTNode<Float> * split();
	void getBasisVector(Float * c);
};

template <typename Float>
inline const QUIC::Uint& CTNode<Float>::operator[](const QUIC::Uint& i) const
{
	return rows[i];
}

template <typename Float>
inline const QUIC::Uint& CTNode<Float>::size() const
{
	return nRows;
}

template <typename Float>
inline void CTNode<Float>::populateChildRows(QUIC::Uint * labels, const QUIC::Uint& blockSize, const QUIC::Uint& nBlocks)
{
	QUIC::Uint rowId = 0;
	QUIC::Uint * leftRow = children[LEFT].rows;
	QUIC::Uint * rightRow = children[RIGHT].rows;
	for (QUIC::Uint i=0; i<nBlocks; ++i)
	{
		QUIC::Uint blockLabels = labels[i];
		QUIC::Uint * blockRows = rows + i*blockSize;
		QUIC::Uint mask = 1;
		for (QUIC::Uint j=0; j<blockSize; ++j)
		{
			if (rowId++ >= nRows) return;
			if (blockLabels & mask) *(leftRow++) = blockRows[j];
			else *(rightRow++) = blockRows[j];
			mask <<= 1uLL;
		}
	}
}

template <typename Float>
inline void CTNode<Float>::initRows()
{
	for (QUIC::Uint i=0; i<nRows; ++i) rows[i] = i;
}

template <typename Float>
inline QUIC::Uint CTNode<Float>::nSamples() const
{
	QUIC::Uint s = ::nSamples(nRows);
	if (s > nRows) s = nRows;
	if (s < 1) s = 1;
	return s;
}

template <typename Float>
inline const LSDistribution<Float>& CTNode<Float>::LS() const
{
	return cosineTree->LS;
}

template <typename Float>
inline const typename Basis<Float>::iterator& CTNode<Float>::getBasisVector() const
{
	assert(basisVector);
	return *basisVector;
}

template <typename Float>
inline void CTNode<Float>::deleteBasisVector()
{
	delete basisVector;
	basisVector = NULL;
}