#pragma once

#include "QUIC.h"
#include <new>

template <typename Float>
class QUIC::MatAllocator
{
public:
	typedef size_t size_type;
	typedef Float * pointer;
	typedef struct {pointer addr; size_type pitch;} pitched_pointer;
	MatAllocator();
	~MatAllocator();
	virtual pitched_pointer allocate(size_type w, size_type h);
	virtual void deallocate(pitched_pointer p, size_type w, size_type h);
	static pitched_pointer make_pitched_pointer(pointer a, size_type p);
};

template <typename Float>
class QUIC::MatAllocatorGPU : public QUIC::MatAllocator<Float>
{
public:
	MatAllocatorGPU();
	~MatAllocatorGPU();
	virtual typename MatAllocator<Float>::pitched_pointer allocate(typename MatAllocator<Float>::size_type w, typename MatAllocator<Float>::size_type h);
	virtual void deallocate(typename MatAllocator<Float>::pitched_pointer p, typename MatAllocator<Float>::size_type w, typename MatAllocator<Float>::size_type h);
};

template <typename Float>
inline typename QUIC::MatAllocator<Float>::pitched_pointer QUIC::MatAllocator<Float>::make_pitched_pointer(pointer a, size_type p)
{
	pitched_pointer pp;
	pp.addr = a;
	pp.pitch = p;
	return pp;
}