#pragma once

#include <map>
#include "QUIC.h"

template <typename Data, typename Key, typename Compare = std::greater<Key> >
class PQueue : public QUIC::Base
{
public:
	PQueue();
	void push(const Data& D, const Key& K);
	Data pop();
	const Key& nextKey();
	bool empty();
private:
	std::multimap<Key, Data, Compare> Q;
};

template <typename Data, typename Key, typename Compare>
inline PQueue<Data, Key, Compare>::PQueue()
: Q()
{}

template <typename Data, typename Key, typename Compare>
inline void PQueue<Data, Key, Compare>::push(const Data& D, const Key& K)
{
	Q.insert(std::make_pair(K, D));
}

template <typename Data, typename Key, typename Compare>
inline Data PQueue<Data, Key, Compare>::pop()
{
	std::multimap<Key, Data, Compare>::iterator beg = Q.begin();
	Data D = beg->second;
	Q.erase(beg);
	return D;
}

template <typename Data, typename Key, typename Compare>
inline const Key& PQueue<Data, Key, Compare>::nextKey()
{
	return Q.begin()->first;
}

template <typename Data, typename Key, typename Compare>
inline bool PQueue<Data, Key, Compare>::empty()
{
	return Q.empty();
}