#pragma once

#include "QUIC.h"
#include "PQueue.h"
#include "CosineTree.h"
#include "PartitionSource.h"

template <typename Float>
class Partition : public QUIC::Base
{
public:
	typedef ::CosineTree<Float> CosineTree;
	Partition(std::ifstream& ifs, const QUIC::Uint& r, const QUIC::Uint& nr, const QUIC::Uint& nc, const Float& e, const Float& d, const QUIC::OrthoMode& om, const QUIC::Uint& mr, const Partition * p);
	Partition(typename QUIC::MatAllocator<Float>::pitched_pointer& data, const QUIC::Uint& r, const QUIC::Uint& nr, const QUIC::Uint& nc, const Float& e, const Float& d, const QUIC::OrthoMode& om, const QUIC::Uint& mr, const Partition * p);
	~Partition();
	virtual void wake();
	virtual void sleep();
	CosineTree * cosineTree;
	QUIC::Matrix<Float> * partition;
	PQueue<CTNode<Float> *, Float> Q;
	typename LSDistribution<Float>::RowSampler * rowSampler;
	QUIC::Uint nRows;
	QUIC::Uint nCols;
	QUIC::Uint run();
	virtual void project(QUIC::Matrix<Float>& basisMatrix, QUIC::Matrix<Float>& projection);
	Float epsilon;
	Float delta;
	QUIC::OrthoMode orthoMode;
	const std::streampos filePos;
	PartitionSource<Float> * source;
	Float mcSqErr;
	Float maxError;
	QUIC::Uint rootNSamples;
	QUIC::Uint r0;
	QUIC::Uint maxRank;
	const Partition * master;
};

template <typename Float>
class PartitionGPU : public Partition<Float>
{
public:
	typedef ::CosineTreeGPU<Float> CosineTree;
	typedef ::BasisGPU<Float> Basis;
	PartitionGPU(std::ifstream& ifs, const QUIC::Uint& r, const QUIC::Uint& nr, const QUIC::Uint& nc, const Float& e, const Float& d, const QUIC::OrthoMode& om, const QUIC::Uint& mr, const PartitionGPU * p);
	PartitionGPU(typename QUIC::MatAllocator<Float>::pitched_pointer& data, const QUIC::Uint& r, const QUIC::Uint& nr, const QUIC::Uint& nc, const Float& e, const Float& d, const QUIC::OrthoMode& om, const QUIC::Uint& mr, const PartitionGPU * p);
	void wake();
	void sleep();
	void project(QUIC::Matrix<Float>& basisMatrix, QUIC::Matrix<Float>& projection);
	static QUIC::Matrix<Float> * fileBuffer;
};