#pragma once

#include <iostream>
#include <string>

namespace QUIC
{
	typedef __int64 Int;
	typedef unsigned __int64 Uint;
	typedef enum {MGS, UGS, BLOCK_MGS} OrthoMode;

	template <typename Float>
	class Matrix;
	template <typename Float>
	class MatrixGPU;

	class Base;
	template <typename T>
	class MatAllocator;
	template <typename T>
	class MatAllocatorGPU;

	// unpartitioned

	template <typename Float>
	QUIC::Uint QUIC_SVD(const Matrix<Float> * A, Matrix<Float> * U, Matrix<Float> * S, Matrix<Float> * VT, const Float& epsilon, const Float& delta, const OrthoMode& orthoMode, const QUIC::Uint& maxRank);
	template <typename Float>
	QUIC::Uint QUIC_SVD(const MatrixGPU<Float> * A, MatrixGPU<Float> * U, MatrixGPU<Float> * S, MatrixGPU<Float> * VT, const Float& epsilon, const Float& delta, const OrthoMode& orthoMode, const QUIC::Uint& maxRank);

	// partitioned, file input

	template <typename Float>
	QUIC::Uint QUIC_SVD(std::ifstream& ist, Matrix<Float> * U, Matrix<Float> * S, Matrix<Float> * VT, const Float& epsilon, const Float& delta, const OrthoMode& orthoMode, const Uint nBlocks, const QUIC::Uint& maxRank);
	template <typename Float>
	QUIC::Uint QUIC_SVD(std::ifstream& ist, MatrixGPU<Float> * U, MatrixGPU<Float> * S, MatrixGPU<Float> * VT, const Float& epsilon, const Float& delta, const OrthoMode& orthoMode, const Uint nBlocks, const QUIC::Uint& maxRank);

	// partitioned, matrix input

	template <typename Float>
	QUIC::Uint QUIC_SVD(const Matrix<Float> * A, Matrix<Float> * U, Matrix<Float> * S, Matrix<Float> * VT, const Float& epsilon, const Float& delta, const OrthoMode& orthoMode, const Uint nBlocks, const QUIC::Uint& maxRank);
	template <typename Float>
	QUIC::Uint QUIC_SVD(const Matrix<Float> * A, MatrixGPU<Float> * U, MatrixGPU<Float> * S, MatrixGPU<Float> * VT, const Float& epsilon, const Float& delta, const OrthoMode& orthoMode, const Uint nBlocks, const QUIC::Uint& maxRank);

	bool initGPU();
	void shutdownGPU();

	void printTimers(std::ostream& ost);
	void resetTimers();

	bool checkError();
	const std::string * getLastError();

	void setBlockSize(const QUIC::Uint& s);
}

#include "Base.h"
#include "Matrix.h"