#pragma once

#define NOMINMAX
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <iostream>
#include <cassert>
#include "QUIC.h"

class Timer
{
public:
	Timer();
	void start();
	void stop();
	void reset();
	void print(std::ostream& ost) const;
	float ms() const;
	bool operator<(const Timer& t) const;
	LARGE_INTEGER ticks;
	LARGE_INTEGER startTime;
	QUIC::Int calls;
	bool started;
};

inline Timer::Timer()
: calls(0), started(false)
{
	startTime.QuadPart = 0;
	ticks.QuadPart = 0;
}

inline void Timer::start()
{
	assert(!started);
	++calls;
	started = true;
	QueryPerformanceCounter(&startTime);
}

inline void Timer::stop()
{
	LARGE_INTEGER stopTime;
	QueryPerformanceCounter(&stopTime);
	assert(started);
	started = false;
	ticks.QuadPart += (stopTime.QuadPart - startTime.QuadPart);
}

inline void Timer::reset()
{
	started = false;
	ticks.QuadPart = 0;
}

inline void Timer::print(std::ostream& ost) const
{
	assert(!started);
	float time = ms();
	ost<<time<<" ms, "<<calls<<" calls";
	if (calls) ost<<", "<<(time/static_cast<float>(calls))<<" ms per call";
	ost<<"\n";
}

inline float Timer::ms() const
{
	LARGE_INTEGER frequency;
	QueryPerformanceFrequency(&frequency);
	float msPeriod = 1000.0f/static_cast<float>(frequency.QuadPart);
	float msTime = ticks.QuadPart*msPeriod;
	return msTime;
}

inline bool Timer::operator<(const Timer& t) const
{
	return ms() < t.ms();
}