#include "profile.h"
#include <cutil_inline.h>

#ifdef PROFILE

#include "Timer.h"

static Timer timers [NUM_TIMER_TYPES][NUM_TIMERS];

#define PRINT_TIMER(id)																						\
if (printCPU && (timers[CPU][id].calls || (printGPU && timers[GPU][id].calls)))								\
{																											\
	ost<<(!printCPU || !printGPU ? "" : timers[CPU][id] < timers[GPU][id] ? "* " : "  ")<<"CPU "<<#id<<" ";	\
	timers[CPU][id].print(ost);																				\
}																											\
if (printGPU && (timers[GPU][id].calls || (printCPU && timers[CPU][id].calls)))								\
{																											\
	ost<<(!printCPU || !printGPU ? "" : timers[GPU][id] < timers[CPU][id] ? "* " : "  ")<<"GPU "<<#id<<" ";	\
	timers[GPU][id].print(ost);																				\
}																																															

void startTimer(const TimerId& id, const TimerType& type)
{
	if (type == GPU) cudaThreadSynchronize();
	timers[type][id].start();
}

void stopTimer(const TimerId& id, const TimerType& type)
{
	if (type == GPU) cudaThreadSynchronize();
	timers[type][id].stop();
}

void printTimers(bool printCPU, bool printGPU, std::ostream& ost)
{
	PRINT_TIMER(PROJECTION);
	PRINT_TIMER(PROJECTION_MUL);
	PRINT_TIMER(PROJECTION_NORM);
	PRINT_TIMER(ROW_NORMS);
	PRINT_TIMER(ROW_MEAN);
	PRINT_TIMER(ORTHOGONALIZATION);
	PRINT_TIMER(COSINES);
	PRINT_TIMER(SVD);
	PRINT_TIMER(SQRT);
	PRINT_TIMER(DIAG_DIV);
	PRINT_TIMER(MUL);
	PRINT_TIMER(COPY_H2D);
	PRINT_TIMER(COPY_D2H);
	PRINT_TIMER(REALLOCATION);
	PRINT_TIMER(BUILDING_TREE);
	PRINT_TIMER(EXTRACTING_SVD);
}

void resetTimers()
{
	for (QUIC::Int i=0; i<NUM_TIMER_TYPES; ++i)
	{
		for (QUIC::Int j=0; j<NUM_TIMERS; ++j)
		{
			timers[i][j].reset();
		}
	}
}

#endif